﻿using FarseerPhysics.Dynamics;
using FarseerPhysics.Collision.Shapes;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FarseerPhysics.Common.Decomposition;
using FarseerPhysics.Common;
using FarseerPhysics.Factories;
using FarseerPhysics.Dynamics.Contacts;

namespace XFORGE.StillCharging
{
    class PlayerShip : Entity
    {
        private float jumpFadeDelay;

        public Turret Turret { get; set; }

        public Shield[] Shields { get; set; }

        private Vector2 desiredDirection;

        public Vector2 DesiredDirection
        {
            get
            {
                return desiredDirection;
            }
            set
            {
                if (value.Length() > 0)
                    desiredDirection = Vector2.Normalize(value);
                else
                    desiredDirection = Vector2.Zero;
            }
        }

        public bool RcsActive { get; set; }

        public bool IsAlive { get; set; }

        public bool IsJumped { get; set; }

        public bool ChargingDrive { get; set; }

        private float driveEnergy;

        public float DriveEnergy
        {
            get
            {
                return driveEnergy;
            }
            set
            {
                driveEnergy = MathHelper.Clamp(value, 0, 1000);
            }
        }

        public PlayerShip(Vector2 position)
            : base(Textures["player_ship"], position)
        {
            DesiredDirection = Vector2.Zero;
            RcsActive = false;
            IsAlive = true;
            IsJumped = false;

            BodyType = BodyType.Dynamic;
            FixedRotation = false;

            AttachTextureFixture(0.01f);

            OnCollision += PlayerShip_OnCollision;

            Turret = new Turret(this, ConvertUnits.ToSimUnits(new Vector2(0, -35.5f)));
            SpawnQueue.Enqueue(Turret);

            Shields = new Shield[4]
                {
                    new Shield(
                        this,
                        ConvertUnits.ToSimUnits(new Vector2(0, -75)),
                        0.0f
                        ),
                    new Shield(
                        this,
                        ConvertUnits.ToSimUnits(new Vector2(75, 0)),
                        MathHelper.PiOver2
                        ),
                    new Shield(
                        this,
                        ConvertUnits.ToSimUnits(new Vector2(0, 75)),
                        MathHelper.Pi
                        ),
                    new Shield(
                        this,
                        ConvertUnits.ToSimUnits(new Vector2(-75, 0)),
                        MathHelper.Pi + MathHelper.PiOver2
                        )
                };

            foreach (var shield in Shields)
            {
                SpawnQueue.Enqueue(shield);
            }
        }

        private bool PlayerShip_OnCollision(Fixture fixtureA, Fixture fixtureB, Contact contact)
        {
            if (!IsJumped)
            {
                IsAlive = false;
                return true;
            }
            else
            {
                return false;
            }
        }

        public override void Update(GameTime gameTime)
        {
            if (IsAlive && !IsJumped)
            {
                if (ChargingDrive)
                {
                    DriveEnergy += 100 * (float)gameTime.ElapsedGameTime.TotalSeconds;

                    ChargeModules(gameTime, -20);
                }
                else
                {
                    DriveEnergy -= 1000 * (float)gameTime.ElapsedGameTime.TotalSeconds;

                    ChargeModules(gameTime, 15 + DriveEnergy);
                }

                ApplyForce(DesiredDirection * (float)gameTime.ElapsedGameTime.TotalSeconds);

                if (RcsActive)
                    ApplyForce(-LinearVelocity * (float)gameTime.ElapsedGameTime.TotalSeconds);
            }
            else
            {
                ChargeModules(gameTime, -25);
            }

            if (IsJumped)
            {
                if (jumpFadeDelay == 0.0f)
                {
                    LinearVelocity = Vector2.Zero;

                    Entity.DespawnQueue.Enqueue(Turret);

                    foreach (var shield in Shields)
                    {
                        Entity.DespawnQueue.Enqueue(shield);
                    }
                }

                jumpFadeDelay += (float)gameTime.ElapsedGameTime.TotalSeconds;

                if (jumpFadeDelay > 1.0f)
                    Entity.DespawnQueue.Enqueue(this);
            }

            base.Update(gameTime);
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            if (!IsJumped)
            {
                base.Draw(spriteBatch);
            }
            else
            {
                spriteBatch.Draw(
                    Textures["jump_flash"],
                    new Rectangle(
                        (int)ConvertUnits.ToDisplayUnits(Position.X),
                        (int)ConvertUnits.ToDisplayUnits(Position.Y),
                        Textures["jump_flash"].Width,
                        Textures["jump_flash"].Height
                        ),
                    null,
                    Color.White * MathHelper.Clamp(1 - jumpFadeDelay, 0.0f, 1.0f),
                    Rotation,
                    new Vector2(
                        Textures["jump_flash"].Width / 2,
                        Textures["jump_flash"].Height / 2
                        ),
                    SpriteEffects.None,
                    0.0f
                    );
            }
        }

        private void ChargeModules(GameTime gameTime, float charge)
        {
            foreach (var shield in Shields)
            {
                shield.Energy += charge * (float)gameTime.ElapsedGameTime.TotalSeconds;
            }

            Turret.Energy += charge * (float)gameTime.ElapsedGameTime.TotalSeconds;
        }
    }
}
