﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace XFORGE.StillCharging
{
    class Starfield
    {
        private const int density = 1;

        private List<Vector2> stars;

        private Texture2D starTinyTexture;
        private Texture2D starMediumTexture;
        private Texture2D pixelTexture;

        private Viewport viewport;

        private Random random;

        private void Recycle(float offset)
        {
            for (int i = 0; i < stars.Count(); i++)
            {
                var star = stars[i];

                if (star.Y > (viewport.Height + offset))

                stars.RemoveAt(i);
            }
        }

        public void Populate(Rectangle area, float offset)
        {
            Recycle(offset);

            for (int i = 0; i < (density * Math.Sqrt(area.Width * area.Height)) ; i++)
            {
                stars.Add(new Vector2(
                    area.X + random.Next(area.Width),
                    area.Y + random.Next(area.Height)
                    ));
            }
        }
        
        public Starfield(Viewport viewport, Dictionary<string, Texture2D> textures)
        {
            this.stars = new List<Vector2>();

            this.starTinyTexture = textures["star_tiny"];
            this.starMediumTexture = textures["star_medium"];
            this.pixelTexture = textures["pixel"];

            this.viewport = viewport;

            this.random = new Random();
        }

        public void Draw(SpriteBatch spriteBatch, float offset)
        {
            spriteBatch.Draw(
                pixelTexture,
                new Rectangle(0, (int)offset - 5, viewport.Width, viewport.Height + 5),
                null,
                Color.Black,
                0.0f,
                Vector2.Zero,
                SpriteEffects.None,
                0.0f
                );

            int i = 0;
            foreach (Vector2 star in stars)
            {
                if (i % 3 == 0)
                {
                    spriteBatch.Draw(
                        starMediumTexture,
                        new Rectangle((int)star.X, (int)star.Y, starMediumTexture.Width, starMediumTexture.Height),
                        null,
                        Color.White,
                        0.0f,
                        new Vector2(starMediumTexture.Width / 2, starMediumTexture.Height / 2),
                        SpriteEffects.None,
                        0.0f
                        );
                }
                else
                {
                    spriteBatch.Draw(
                        starTinyTexture,
                        new Rectangle((int)star.X, (int)star.Y, starTinyTexture.Width, starTinyTexture.Height),
                        null,
                        Color.White,
                        0.0f,
                        new Vector2(starTinyTexture.Width / 2, starTinyTexture.Height / 2),
                        SpriteEffects.None,
                        0.0f
                        );
                }

                i++;
            }
        }
    }
}
