﻿using FarseerPhysics.Dynamics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace XFORGE.StillCharging
{
    class Turret : SubEntity
    {
        public Vector2 Target;

        private float energy;

        public float Energy
        {
            get
            {
                return energy;
            }
            set
            {
                energy = MathHelper.Clamp(value, 0, 25);
            }
        }

        public Turret(Entity parent, Vector2 offset)
            : base(Textures["turret_small"], parent, offset, 0.0f)
        {
            Target = Vector2.Zero;

            Energy = 25;

            this.origin = new Vector2(7.5f, 10.5f);

            BodyType = BodyType.Static;
        }

        public void Fire()
        {
            if (Energy >= 5.0f)
            {
                Energy -= 5.0f;

                var direction = Vector2.Transform(
                    ConvertUnits.ToSimUnits(new Vector2(0, -10)),
                    Matrix.CreateRotationZ(Rotation)
                    );

                var bolt = new LaserBolt(
                    Position + direction,
                    Rotation
                    );

                bolt.IgnoreCollisionWith(this);
                bolt.IgnoreCollisionWith(parent);

                direction.Normalize();

                bolt.ApplyLinearImpulse(direction * 5 + parent.LinearVelocity);

                SpawnQueue.Enqueue(bolt);
            }
        }

        public override void Update(GameTime gameTime)
        {
            if (Energy > 5.0f)
            {
                var angleToTarget = MathHelper.WrapAngle((float)Math.Atan2(
                    Position.Y - Target.Y,
                    Position.X - Target.X
                    ));

                var differenceToTarget = MathHelper.WrapAngle(angleToTarget - Rotation - MathHelper.PiOver2);

                Rotation += (float)Math.Tanh(differenceToTarget) * 3 * (float)gameTime.ElapsedGameTime.TotalSeconds;
            }

            base.Update(gameTime);
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            base.Draw(spriteBatch);
        }
    }
}
